# robobin/message_handler.py
import time
class MessageHandler:
    def __init__(self, api_node, testing=False):
        self.api_node = api_node
        self.testing = testing
        self.handlers = {
            "PING": self.handle_ping,
            "TIME": self.handle_time_request,
            "MANUALCTRL": self.handle_manual_control
        }

    def handle_message(self, client_socket, raw_message):
        """Parses the incoming message and routes the command to the appropriate handler."""
        command, *args = raw_message.split(" ", 1)
        data = args[0] if args else None
        handler = self.handlers.get(command, self.handle_unknown_message)
        return handler(client_socket, data)

    def handle_ping(self, client_socket, _):
        """Responds with a PONG message."""
        response = b"PONG"
        if self.testing:
            print(response.decode())
        else:
            client_socket.sendall(response)

        return None

    def handle_time_request(self, client_socket, _):
        """Sends the current server time."""
        response = time.ctime().encode()
        if self.testing:
            print(response.decode())
        else:
            client_socket.sendall(response)

        return None
    
    def handle_manual_control(self, client_socket, message):
        """Handles manual control commands: W, A, S, D."""
        # W: linear.x = 0.5, angular.z = 0
        # A: linear.x = 0, angular.z = 0.5
        # S: linear.x = -0.5, angular.z = 0
        # D: linear.x = 0, angular.z = -0.5
        
        directions = {
            "W": (0.5, 0),    # Move forward
            "A": (0, 0.5),    # Turn left
            "S": (-0.5, 0),   # Move backward
            "D": (0, -0.5)    # Turn right
        }
        response_data = directions.get(message.strip().upper(), (0, 0))
        response = f"Manual control command received: {response_data}".encode()
        if self.testing:
            print(response.decode())
        else:
            client_socket.sendall(response)
        print("Processed manual control command:", response_data)
        return "cmd_vel", response_data
    
    def handle_unknown_message(self, client_socket, _):
        """Handles unknown commands."""
        response = b"Unknown command"
        if self.testing:
            print(response.decode())
        else:
            client_socket.sendall(response)
        return None
    
# Test class without api_node and with testing enabled
if __name__ == "__main__":
    message_handler = MessageHandler(None, testing=True)
    assert message_handler.handle_message(None, "PING") is None
    assert message_handler.handle_message(None, "TIME") is None
    assert message_handler.handle_message(None, "MANUALCTRL W") == ("cmd_vel", (0.5, 0))
    assert message_handler.handle_message(None, "MANUALCTRL A") == ("cmd_vel", (0, 0.5))
    assert message_handler.handle_message(None, "MANUALCTRL S") == ("cmd_vel", (-0.5, 0))
    assert message_handler.handle_message(None, "MANUALCTRL D") == ("cmd_vel", (0, -0.5))
    assert message_handler.handle_message(None, "UNKNOWN") is None